"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = __importDefault(require("path"));
var fs_1 = __importDefault(require("fs"));
var utils_1 = require("../shared/utils");
/**
 * Adds service packing support
 */
var PackageService = /** @class */ (function () {
    function PackageService(serverless) {
        this.serverless = serverless;
    }
    /**
     * Creates the function.json binding files required for the serverless service
     */
    PackageService.prototype.createBindings = function () {
        var _this = this;
        var createEventsPromises = this.serverless.service.getAllFunctions()
            .map(function (functionName) {
            var metaData = utils_1.Utils.getFunctionMetaData(functionName, _this.serverless);
            return _this.createBinding(functionName, metaData);
        });
        return Promise.all(createEventsPromises);
    };
    /**
     * Prepares a serverless project for webpack and copies required files including
     * host.json and function.json files
     */
    PackageService.prototype.prepareWebpack = function () {
        var _this = this;
        var filesToCopy = [];
        if (fs_1.default.existsSync("host.json")) {
            filesToCopy.push("host.json");
        }
        this.serverless.service.getAllFunctions().forEach(function (functionName) {
            var functionJsonPath = path_1.default.join(functionName, "function.json");
            if (fs_1.default.existsSync(functionJsonPath)) {
                filesToCopy.push(functionJsonPath);
            }
        });
        this.serverless.cli.log("Copying files for webpack");
        filesToCopy.forEach(function (filePath) {
            var destinationPath = path_1.default.join(".webpack", "service", filePath);
            var destinationDirectory = path_1.default.dirname(destinationPath);
            if (!fs_1.default.existsSync(destinationDirectory)) {
                fs_1.default.mkdirSync(destinationDirectory);
            }
            fs_1.default.copyFileSync(filePath, destinationPath);
            _this.serverless.cli.log("-> " + destinationPath);
        });
        return Promise.resolve();
    };
    /**
     * Cleans up generated function.json files after packaging has completed
     */
    PackageService.prototype.cleanUp = function () {
        this.serverless.service.getAllFunctions().map(function (functionName) {
            // Delete function.json if exists in function folder
            var filePath = path_1.default.join(functionName, "function.json");
            if (fs_1.default.existsSync(filePath)) {
                fs_1.default.unlinkSync(filePath);
            }
            // Delete function folder if empty
            var items = fs_1.default.readdirSync(functionName);
            if (items.length === 0) {
                fs_1.default.rmdirSync(functionName);
            }
        });
        return Promise.resolve();
    };
    /**
     * Creates the function.json for for the specified function
     */
    PackageService.prototype.createBinding = function (functionName, functionMetadata) {
        var functionJSON = functionMetadata.params.functionsJson;
        functionJSON.entryPoint = functionMetadata.entryPoint;
        functionJSON.scriptFile = functionMetadata.handlerPath;
        var functionDirPath = path_1.default.join(this.serverless.config.servicePath, functionName);
        if (!fs_1.default.existsSync(functionDirPath)) {
            fs_1.default.mkdirSync(functionDirPath);
        }
        fs_1.default.writeFileSync(path_1.default.join(functionDirPath, "function.json"), JSON.stringify(functionJSON, null, 2));
        return Promise.resolve();
    };
    return PackageService;
}());
exports.PackageService = PackageService;
